from __future__ import unicode_literals

import json
from random import randint
from django.core.urlresolvers import reverse_lazy
from .utils import AnonymizerServiceTestUtils
from . import AnonymizerServiceTestCase


class UpdateVotesTest(AnonymizerServiceTestUtils,
                      AnonymizerServiceTestCase):
    """
    Class to test the update a list of votes
    """
    # Setup the Test Case
    def setUp(self):
        """ Setup all needed endpoints """
        # Random election ID
        self.dummy_election_id = str(randint(1, 10))
        self.another_election_id = str(20)
        req_kwargs = {'election_id': self.dummy_election_id}
        req_kwargs_second = {'election_id': self.another_election_id}
        # Initialize election URL
        self.init_url = reverse_lazy('api:initializeelection',
                                     kwargs=req_kwargs)
        # Another initialize election URL
        self.init_url_other = reverse_lazy(
            'api:initializeelection',
            kwargs=req_kwargs_second)
        # Reference list URL
        self.update_url = reverse_lazy(
            'api:deciphervotes',
            kwargs=req_kwargs)

    def test_updatevotes_get_method(self):
        """ Test for method get not allowed """
        r = self.client.get(self.update_url)
        self.assertEqual(r.status_code, 405)

    def test_updatevotes_post_method(self):
        """ Test empty list of references for non-existing election """
        r = self.client.post(self.update_url)
        self.assertEqual(r.status_code, 400)

    def test_updatevotes_success(self):
        """ Test updating existing votes """
        # Initialize election
        init_obj = self.create_init(3)
        r = self.client.post(self.init_url, init_obj)

        # Update the two first votes inserted
        post_data = {
            "vote_list": [
                {"id": "1", "deciphered_vote": "dummyvote"},
                {"id": "2", "deciphered_vote": "dummyvote2"},
             ]
        }

        # Post update data
        r = self.client.post(self.update_url,
                             json.dumps(post_data),
                             content_type='application/json')
        self.assertEqual(r.status_code, 200)
        response = r.content.decode('string-escape').strip('"')
        json_response = json.loads(response)

        # Check if both votes where successfuly updated
        self.assertIn("success", json_response)
        self.assertEqual(len(json_response["success"]), 2)

    def test_updatevotes_failed(self):
        """ Test failed updating a non-existing vote entry """
        # Initialize election
        init_obj = self.create_init(3)
        r = self.client.post(self.init_url, init_obj)

        # Update the two first votes inserted
        post_data = {
            "vote_list": [
                {"id": "100", "deciphered_vote": "dummyvote"}
             ]
        }

        # Post update data
        r = self.client.post(self.update_url,
                             json.dumps(post_data),
                             content_type='application/json')
        self.assertEqual(r.status_code, 200)
        response = r.content.decode('string-escape').strip('"')
        json_response = json.loads(response)

        # Check if vote was marked as failed
        self.assertIn("failed", json_response)
        self.assertEqual(len(json_response["failed"]), 1)

    def test_updatevotes_invalid(self):
        """ Test failed updating an invalid vote entry """
        # Initialize election
        init_obj = self.create_init(3)
        r = self.client.post(self.init_url, init_obj)

        # Update mal-formed request data
        post_data = {
            "vote_list": [
                {"deciphered_vote": "dummyvote"},
                {"id": "100"}
             ]
        }

        # Post update data
        r = self.client.post(self.update_url,
                             json.dumps(post_data),
                             content_type='application/json')
        self.assertEqual(r.status_code, 200)
        response = r.content.decode('string-escape').strip('"')
        json_response = json.loads(response)

        # Check if vote without deciphered_vote was marked as invalid
        self.assertIn("invalid", json_response)
        self.assertEqual(json_response["invalid"], 1)
        # Check if vote without id was marked as failed
        self.assertIn("failed", json_response)
        self.assertEqual(len(json_response["failed"]), 1)

    def test_updatevotes_mismatch(self):
        """ Test failed updating a vote with invalid election id """
        # Initialize election
        init_obj = self.create_init(3)
        r = self.client.post(self.init_url_other, init_obj)
        r = self.client.post(self.init_url, init_obj)

        # Update mal-formed request data
        post_data = {
            "vote_list": [
                {"id": "1", "deciphered_vote": "dummyvote"}
             ]
        }

        # Post update data
        r = self.client.post(self.update_url,
                             json.dumps(post_data),
                             content_type='application/json')
        self.assertEqual(r.status_code, 200)

        response = r.content.decode('string-escape').strip('"')
        json_response = json.loads(response)

        # Check if vote was marked as failed
        self.assertIn("failed", json_response)
        self.assertEqual(len(json_response["failed"]), 1)


class CountVotesTest(AnonymizerServiceTestUtils,
                     AnonymizerServiceTestCase):
    """
    Class to test the partial vote count
    """
    # Setup the Test Case
    def setUp(self):
        """ Setup all needed endpoints """
        # Random election ID
        self.dummy_election_id = str(randint(1, 10))
        req_kwargs = {'election_id': self.dummy_election_id}
        # Initialize election URL
        self.init_url = reverse_lazy('api:initializeelection',
                                     kwargs=req_kwargs)
        # Reference list URL
        self.count_url = reverse_lazy('api:partialcount',
                                      kwargs=req_kwargs)

    def test_updatevotes_post_method(self):
        """ Test for method get not allowed """
        r = self.client.post(self.count_url)
        self.assertEqual(r.status_code, 405)

    def test_updatevotes_get_method(self):
        """ Test empty list of references for non-existing election """
        r = self.client.get(self.count_url)
        self.assertEqual(r.status_code, 200)
